==> libjabber-0.9RC.tar.gz: libjabber-0.9RC/src/jpacket.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"

jpacket jpacket_new(xmlnode x)
{
    jpacket p;

    p = pmalloc(xmlnode_pool(x),sizeof(_jpacket));
    p->x = x;

    return jpacket_reset(p);
}

jpacket jpacket_reset(jpacket p)
{
    char *val;
    xmlnode x;

    x = p->x;
    memset(p,0,sizeof(_jpacket));
    p->x = x;
    p->p = xmlnode_pool(x);

    if(*(xmlnode_get_name(x)) == 'm')
    {
        p->type = JPACKET_MESSAGE;
    }else if(*(xmlnode_get_name(x)) == 'p')
    {
        p->type = JPACKET_PRESENCE;
        val = xmlnode_get_attrib(x, "type");
        if(val == NULL)
            p->subtype = JPACKET__AVAILABLE;
        else if(j_strcmp(val,"unavailable") == 0)
            p->subtype = JPACKET__UNAVAILABLE;
        else if(j_strcmp(val,"probe") == 0)
            p->subtype = JPACKET__PROBE;
        else if(*val == 's' || *val == 'u')
            p->type = JPACKET_S10N;
        else
            p->type = JPACKET_UNKNOWN;
    }else if(*(xmlnode_get_name(x)) == 'i')
    {
        p->type = JPACKET_IQ;
        p->iq = xmlnode_get_tag(x,"?xmlns");
        p->iqns = xmlnode_get_attrib(p->iq,"xmlns");
    }

    val = xmlnode_get_attrib(x,"to");
    if(val != NULL)
        p->to = jid_new(p->p, val);
    val = xmlnode_get_attrib(x,"from");
    if(val != NULL)
        p->from = jid_new(p->p, val);

    return p;
}


int jpacket_subtype(jpacket p)
{
    char *type;
    int ret = p->subtype;

    if(ret != JPACKET__UNKNOWN)
        return ret;

    type = xmlnode_get_attrib(p->x, "type");
    if(type == NULL)
        ret = JPACKET__NONE;
    else if(j_strcmp(type,"error") == 0)
        ret = JPACKET__ERROR;
    else
        switch(p->type)
        {
            case JPACKET_MESSAGE:
                if(j_strcmp(type,"chat") == 0)
                    ret = JPACKET__CHAT;
                else if(j_strcmp(type,"groupchat") == 0)
                    ret = JPACKET__GROUPCHAT;
                break;
            case JPACKET_S10N:
                if(j_strcmp(type,"subscribe") == 0)
                    ret = JPACKET__SUBSCRIBE;
                else if(j_strcmp(type,"subscribed") == 0)
                    ret = JPACKET__SUBSCRIBED;
                else if(j_strcmp(type,"unsubscribe") == 0)
                    ret = JPACKET__UNSUBSCRIBE;
                else if(j_strcmp(type,"unsubscribed") == 0)
                    ret = JPACKET__UNSUBSCRIBED;
                break;
            case JPACKET_IQ:
                if(j_strcmp(type,"get") == 0)
                    ret = JPACKET__GET;
                else if(j_strcmp(type,"set") == 0)
                    ret = JPACKET__SET;
                else if(j_strcmp(type,"result") == 0)
                    ret = JPACKET__RESULT;
                break;
        }

    p->subtype = ret;
    return ret;
}

==> libjabber-1.0RC.tar.gz: libjabber-1.0RC/src/jpacket.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"

jpacket jpacket_new(xmlnode x)
{
    jpacket p;

    p = pmalloc(xmlnode_pool(x),sizeof(_jpacket));
    p->x = x;

    return jpacket_reset(p);
}

jpacket jpacket_reset(jpacket p)
{
    char *val;
    xmlnode x;

    x = p->x;
    memset(p,0,sizeof(_jpacket));
    p->x = x;
    p->p = xmlnode_pool(x);

    if(*(xmlnode_get_name(x)) == 'm')
    {
        p->type = JPACKET_MESSAGE;
    }else if(*(xmlnode_get_name(x)) == 'p')
    {
        p->type = JPACKET_PRESENCE;
        val = xmlnode_get_attrib(x, "type");
        if(val == NULL)
            p->subtype = JPACKET__AVAILABLE;
        else if(strcmp(val,"unavailable") == 0)
            p->subtype = JPACKET__UNAVAILABLE;
        else if(strcmp(val,"probe") == 0)
            p->subtype = JPACKET__PROBE;
        else if(*val == 's' || *val == 'u')
            p->type = JPACKET_S10N;
        else if(strcmp(val,"available") == 0)
        { /* someone is using type='available' which is frowned upon */
            xmlnode_hide_attrib(x,"type");
            p->subtype = JPACKET__AVAILABLE;
        }else
            p->type = JPACKET_UNKNOWN;
    }else if(*(xmlnode_get_name(x)) == 'i')
    {
        p->type = JPACKET_IQ;
        p->iq = xmlnode_get_tag(x,"?xmlns");
        p->iqns = xmlnode_get_attrib(p->iq,"xmlns");
    }

    val = xmlnode_get_attrib(x,"to");
    if(val != NULL)
        p->to = jid_new(p->p, val);
    val = xmlnode_get_attrib(x,"from");
    if(val != NULL)
        p->from = jid_new(p->p, val);

    return p;
}


int jpacket_subtype(jpacket p)
{
    char *type;
    int ret = p->subtype;

    if(ret != JPACKET__UNKNOWN)
        return ret;

    type = xmlnode_get_attrib(p->x, "type");
    if(type == NULL)
        ret = JPACKET__NONE;
    else if(j_strcmp(type,"error") == 0)
        ret = JPACKET__ERROR;
    else
        switch(p->type)
        {
            case JPACKET_MESSAGE:
                if(j_strcmp(type,"chat") == 0)
                    ret = JPACKET__CHAT;
                else if(j_strcmp(type,"groupchat") == 0)
                    ret = JPACKET__GROUPCHAT;
                break;
            case JPACKET_S10N:
                if(j_strcmp(type,"subscribe") == 0)
                    ret = JPACKET__SUBSCRIBE;
                else if(j_strcmp(type,"subscribed") == 0)
                    ret = JPACKET__SUBSCRIBED;
                else if(j_strcmp(type,"unsubscribe") == 0)
                    ret = JPACKET__UNSUBSCRIBE;
                else if(j_strcmp(type,"unsubscribed") == 0)
                    ret = JPACKET__UNSUBSCRIBED;
                break;
            case JPACKET_IQ:
                if(j_strcmp(type,"get") == 0)
                    ret = JPACKET__GET;
                else if(j_strcmp(type,"set") == 0)
                    ret = JPACKET__SET;
                else if(j_strcmp(type,"result") == 0)
                    ret = JPACKET__RESULT;
                break;
        }

    p->subtype = ret;
    return ret;
}
